#include "gesturewidget.h"
#include "textlabel.h"
#include <QString>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPixmap>
#include <QIcon>
#include <QFocusEvent>
#include <QByteArray>
#include <QPainter>
#include <QDebug>
#include <QApplication>
#include <QGSettings>
#include <qt5-ukui/highlight-effect.h>

static const char* IconName = "video-x-generic-symbolic";
static const char* ThemeSchemas = "org.ukui.style";

QPixmap highLightPixmap(QPixmap pixmap);

GestureWidget::GestureWidget(QWidget *parent) : QPushButton(parent)
{
    initProperty();

    m_gestureDescription = new TextLabel(this);
    m_gestureAction = new TextLabel(this);
    m_playIcon = createPlayIconLabel();
    setActionTextDefaultColor();

    m_gestureDescriptionLayout = createGestureNameLayout();
    m_mainLayout = createMainLayout();

    initConnections();
}

void GestureWidget::themeChanged()
{
    if(QIcon::hasThemeIcon(IconName)) {
        QIcon play = QIcon::fromTheme(IconName);
        m_playIcon->setPixmap(play.pixmap(QSize(16, 16)));
    }
}

void GestureWidget::setGestureDescription(const QString &text)
{
    m_gestureDescription->setText(text);
    m_gestureDescription->setRealText(text);
    updateActionMaxWidth();
}

void GestureWidget::setGestureAction(const QString &text)
{
    m_gestureAction->setText(text);
    m_gestureAction->setRealText(text);
}

void GestureWidget::focusOutEvent(QFocusEvent *e)
{
    if(e->reason() == Qt::FocusReason::MouseFocusReason) {
        emit loseFocus();
    }

    QPushButton::focusOutEvent(e);
}

void GestureWidget::onToggled(bool checked)
{
    //! \todo Encapsulate all the logic instead of using 'setStyleSheet'
    if (checked == true) {
        m_gestureDescription->setStyleSheet("QLabel{ color: white }");

        setActionTextCheckedColor();
    } else {
        m_gestureDescription->setStyleSheet("");

        setActionTextDefaultColor();
    }
}

void GestureWidget::updateActionMaxWidth()
{
    QFontMetrics metrics(qApp->font());
    int textWidth = metrics.width(m_gestureDescription->realText());
    m_gestureDescription->setMaximumWidth(textWidth);
    m_gestureDescription->updateElidedText();
}

void GestureWidget::initProperty()
{
    setFocusPolicy(Qt::FocusPolicy::ClickFocus);
    setFixedHeight(64);
    setCheckable(true);
    setAutoExclusive(true);
    setProperty("useButtonPalette", true);
    //统一使用主题的处理机制
    setFlat(true);
}

void GestureWidget::initConnections()
{
    //由于button中增加的QLabel没有办法实现反白效果，需要自己单独处理设置颜色
    connect(this,SIGNAL(toggled(bool)),this,SLOT(onToggled(bool)));

    connect(qApp, &QApplication::fontChanged, this, [this](){
        updateActionMaxWidth();
        m_gestureAction->updateElidedText();
    });

    QGSettings *styleSettings  = new QGSettings(ThemeSchemas, QByteArray(), this);
    connect(styleSettings, &QGSettings::changed, this, [this](const QString& key) {
        if ("styleName" != key) {
            return;
        }
        if (this->isChecked())
            setActionTextCheckedColor();
        else
            setActionTextDefaultColor();
    });
}

QLabel* GestureWidget::createPlayIconLabel()
{
    QLabel* playIcon = new QLabel(this);

    playIcon->setStyle(IconLabelStyle::getStyle());
    playIcon->setObjectName("IconLabel");

    playIcon->setAlignment(Qt::AlignVCenter);
    // 上边距设为 1 是为了让图标居中，可能是 QT 的 bug
    playIcon->setContentsMargins(0, 1, 0, 0);

    if(QIcon::hasThemeIcon(IconName)) {
        QIcon play = QIcon::fromTheme(IconName);
        playIcon->setPixmap(play.pixmap(QSize(16, 16)));
    }

    return playIcon;
}

QHBoxLayout* GestureWidget::createGestureNameLayout()
{
    Q_ASSERT(m_playIcon);
    Q_ASSERT(m_gestureDescription);

    QHBoxLayout *hlayout = new QHBoxLayout;
    hlayout->setContentsMargins(0, 0, 0, 0);
    hlayout->setSpacing(8);
    hlayout->addWidget(m_gestureDescription);
    hlayout->addWidget(m_playIcon, 0, Qt::AlignVCenter);

    return hlayout;
}

QVBoxLayout* GestureWidget::createMainLayout()
{
    Q_ASSERT(m_gestureDescriptionLayout);
    Q_ASSERT(m_gestureAction);

    QVBoxLayout *vlayout = new QVBoxLayout(this);
    vlayout->setContentsMargins(16, 10, 16, 7);
    vlayout->setSpacing(0);
    vlayout->addLayout(m_gestureDescriptionLayout);
    vlayout->addWidget(m_gestureAction);

    return vlayout;
}

QString GestureWidget::placeholderTextColor() const
{
    QPalette pal;
    QColor textColor = pal.placeholderText().color();
    QString rgbaString = QString("color: rgba(%1,%2,%3,%4)")
           .arg(textColor.red())
           .arg(textColor.green())
           .arg(textColor.blue())
           .arg(textColor.alphaF());
    return rgbaString;
}

QString GestureWidget::placeholderTextCheckedColor() const
{
    //! \todo It should be a dynamic value, not a fixed value as it is now
    return QString("color: rgba(255,255,255,0.34902)");
}

//! \note ukui-platformtheme supports the placeholderText color of Label,
//! but as a composite control, the internal QLabel will not change
//! the color when hover and checked
void GestureWidget::setActionTextDefaultColor()
{
    Q_ASSERT(m_gestureAction);

    m_gestureAction->setStyleSheet(placeholderTextColor());
}

void GestureWidget::setActionTextCheckedColor()
{
    Q_ASSERT(m_gestureAction);

    m_gestureAction->setStyleSheet(placeholderTextCheckedColor());
}

// IconLabelStyle
static IconLabelStyle* global_instance = nullptr;
IconLabelStyle* IconLabelStyle::getStyle()
{
    if (!global_instance)
        global_instance = new IconLabelStyle;
    return global_instance;
}

void IconLabelStyle::drawControl(ControlElement element, const QStyleOption *option,
                                 QPainter *painter, const QWidget *widget) const
{
    if (widget->objectName() != "IconLabel") {
        return QProxyStyle::drawControl(element, option, painter, widget);
    }

    QIcon play = QIcon::fromTheme(IconName);
    QPixmap pixmap = HighLightEffect::ordinaryGeneratePixmap(play.pixmap(QSize(16, 16)),
                                                             option, widget);

    // 设计要求在控件被checked的时候也要反白，但是被checked的是外层的button控件而非这里
    // 因此我们需要单独获取外层button的isChecked，来进行反白
    QPushButton* parent = static_cast<QPushButton*>(widget->parent());
    if (parent && parent->isChecked()) {
        pixmap = highLightPixmap(pixmap);
    }

    // QLabel的复制构造被删除了，因此不能构造一个新的对象来修改，只能转换成非const类型
    const QLabel* label = static_cast<const QLabel*>(widget);
    QLabel* iconLabel = const_cast<QLabel*>(label);
    iconLabel->setPixmap(pixmap);
    QProxyStyle::drawControl(element, option, painter, iconLabel);
}

QPixmap highLightPixmap(QPixmap pixmap)
{
    QPixmap target = pixmap;
    QPainter p(&target);
    p.setRenderHint(QPainter::Antialiasing);
    p.setRenderHint(QPainter::SmoothPixmapTransform);
    p.setCompositionMode(QPainter::CompositionMode_SourceIn);
    p.fillRect(target.rect(), QColor(Qt::white));

    return target;
}
