#include "touchscreen.h"
#include "ui_touchscreen.h"

#include "gesturewidget.h"

#include <kysdk/applications/kswitchbutton.h>
#include <ukuisdk/kylin-com4cxx.h>

#include <QVBoxLayout>
#include <QUrl>
#include <QDebug>
#include <QResizeEvent>
#include <QPalette>
#include <QPushButton>
#include <QDBusInterface>
#include <QDBusReply>
#include <QMovie>

#include <unistd.h>

const quint32 maxCycleNum = 3;
const QByteArray themeSchemas = "org.ukui.style";
const QString iconThemeName = "icon-theme-name";

#define KYLIN_USER_GUIDE_PATH              "/"
#define KYLIN_USER_GUIDE_SERVICE           "com.kylinUserGuide.hotel"
#define KYLIN_USER_GUIDE_INTERFACE         "com.guide.hotel"

TouchScreen::TouchScreen(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::TouchScreen)
    , m_cycleNum(0)
    , m_iconThemeGSettings(nullptr)
{
    m_productFeat = QString::fromStdString(KDKGetOSRelease("PRODUCT_FEATURES")).toInt();
    ui->setupUi(this);
    initUI();
    loadGif();
    monitorIconThemeChange();
}

TouchScreen::~TouchScreen()
{
    delete ui;
    if(m_iconThemeGSettings) {
        m_iconThemeGSettings->deleteLater();
        m_iconThemeGSettings = nullptr;
    }
}

bool TouchScreen::eventFilter(QObject *watched, QEvent *event)
{
//    if (watched == videoWidget) {
//        qDebug() << __func__ << event;
//    }
    return eventFilter(watched, event);
}

void TouchScreen::monitorIconThemeChange()
{
    QByteArray themeId(themeSchemas);
    if (QGSettings::isSchemaInstalled(themeId)) {
        m_iconThemeGSettings = new QGSettings(themeId);
        connect(m_iconThemeGSettings, &QGSettings::changed, this, [=](const QString &key) {
            if (key == "iconThemeName") {
                emit iconThemeChangedSignal();
            }
        });
    }
}

void TouchScreen::initUI()
{

//    QFont font;
//    font.setPointSize(14);
//    ui->touchscreen_label->setFont(font);

//    QPalette paltte = this->palette();
//    paltte.setColor(QPalette::Window, paltte.base().color());
//    setPalette(paltte);


//    setAttribute(Qt::WA_TranslucentBackground, true);
//    setStyleSheet("background:transparent");
//    ui->moreGestureButton->setProperty("useButtonPalette", true);
//    ui->moreGestureButton->setStyleSheet("QPushButton:!checked{background-color: palette(base)}");

    //~ contents_path /TouchScreen/touchscreen gesture
    ui->touchscreen_label->setText(tr("touchscreen gesture"));
    //~ contents_path /TouchScreen/more gesture
    ui->moreGestureButton->setText(tr("more gesture"));
    //~ contents_path /TouchScreen/Tablet Mode
    ui->switchTablet_label->setText(tr("Tablet Mode"));

    QHBoxLayout* switchTabletLayout = new QHBoxLayout();
    ui->switchTablet_frame->setFrameShape(QFrame::Shape::Box);
    ui->switchTablet_frame->setLayout(switchTabletLayout);
    QLabel* autoSwtich = new QLabel(tr("auto switch tablet mode"));
    switchTabletLayout->addWidget(autoSwtich);
    switchTabletLayout->addStretch();
    switchTabletLayout->setContentsMargins(16, 0, 16, 0);
    m_autoSwitchTablet = new kdk::KSwitchButton();
    m_autoSwitchTablet->setChecked(this->autoSwitchTablet());
    switchTabletLayout->addWidget(m_autoSwitchTablet);
    connect(m_autoSwitchTablet, &kdk::KSwitchButton::clicked, this, [this](bool checked) {
        if (checked) {
            setAutoSwitchTablet(true);
        }
        else {
            setAutoSwitchTablet(false);
        }
    });

    QDBusConnection::sessionBus().connect("com.kylin.statusmanager.interface",
                                          "/",
                                          "com.kylin.statusmanager.interface",
                                          "modemonitor_change_signal",
                                          this,
                                          SLOT(setAutoSwitchTabletBtn(bool)));

    //! \note Do not show tablet mode if os without tablet mode
    if (m_productFeat != 3) {
        ui->switchTablet_label->hide();
        ui->switchTablet_frame->hide();
        ui->horizontalLayout_4->deleteLater();
    }

    QVBoxLayout *vlayout = new QVBoxLayout(ui->gesture_widget);
    vlayout->setMargin(0);
    vlayout->setContentsMargins(0, 0, 0, 0);
    vlayout->setSpacing(4);

    GestureWidget *buttomSwipe = new GestureWidget(ui->gesture_widget);
    buttomSwipe->setGestureDescription(tr("swipe up from the bottom edge"));
    buttomSwipe->setGestureAction(tr("open the multitasking view"));

    GestureWidget *rightSwipe = new GestureWidget(ui->gesture_widget);
    rightSwipe->setGestureDescription(tr("swipe left from the right edge"));
    rightSwipe->setGestureAction(tr("show sidebar"));

    GestureWidget *downSwipe = new GestureWidget(ui->gesture_widget);
    downSwipe->setGestureDescription(tr("swipe four fingers down anywhere"));
    downSwipe->setGestureAction(tr("show global search"));

    GestureWidget *horizenSwipe = new GestureWidget(ui->gesture_widget);
    horizenSwipe->setGestureDescription(tr("swipe four fingers horizontally"));
    horizenSwipe->setGestureAction(tr("switch windows"));

    m_gestureArray.append(buttomSwipe);
    m_gestureArray.append(rightSwipe);
    m_gestureArray.append(downSwipe);
    m_gestureArray.append(horizenSwipe);
    ui->gesture_widget->setLayout(vlayout);
    vlayout->addWidget(buttomSwipe);
    vlayout->addWidget(rightSwipe);
    vlayout->addWidget(downSwipe);
    vlayout->addWidget(horizenSwipe);
    connect(buttomSwipe, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::bottomEdgeSwipe);
    });
    connect(rightSwipe, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::rightEdgeSwipe);
    });
    connect(downSwipe, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::fourFingersSwipeDown);
    });
    connect(horizenSwipe, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::fourFingersHorizontalSwipe);
    });
    connect(ui->moreGestureButton, &QPushButton::clicked, this, [=]() {
        QString pluName = "ukui-control-center/TouchScreen";
        qDebug() << pluName;
        QString service_name = "com.kylinUserGuide.hotel_" + QString::number(getuid());
        QDBusInterface interface(service_name,
                                 KYLIN_USER_GUIDE_PATH,
                                 KYLIN_USER_GUIDE_INTERFACE,
                                 QDBusConnection::sessionBus(),
                                 this);
        QDBusMessage msg = interface.call("showGuide", pluName);
    });

    foreach (auto gesture, m_gestureArray) {
        connect(gesture, &GestureWidget::loseFocus, this, &TouchScreen::stopGif);
        connect(this, &TouchScreen::iconThemeChangedSignal, gesture, &GestureWidget::themeChanged);
    }
//    connect(buttomSwipe, &GestureWidget::loseFocus, this, &TouchScreen::stopGif);
//    connect(topSwipe, &GestureWidget::loseFocus, this, &TouchScreen::stopGif);
//    connect(rightSwipe, &GestureWidget::loseFocus, this, &TouchScreen::stopGif);
//    connect(downSwipe, &GestureWidget::loseFocus, this, &TouchScreen::stopGif);
//    connect(horizenSwipe, &GestureWidget::loseFocus, this, &TouchScreen::stopGif);
}

void TouchScreen::loadGif()
{
//    ui->video_widget->setStyleSheet("border-image: none; background: green; border:none");
    QLabel *video_label = new QLabel(ui->video_widget);
    video_label->setFixedSize(400, 240);
//    video_label->setStyleSheet("border-image: none; background: yellow; border:none");
    QVBoxLayout *videoLayout = new QVBoxLayout(ui->video_widget);
    videoLayout->setContentsMargins(0, 0, 16, 0);
    videoLayout->addWidget(video_label);
    m_movie = new QMovie;
    m_movie->setFileName(":/gif/resources/下边缘上划显示多任务视图.gif");
//    m_movie->setScaledSize(QSize(384, 250));
    m_movie->setScaledSize(QSize(385, 231));
    m_movie->start();
    m_movie->stop();
    video_label->setMovie(m_movie);

//    QObject::connect(m_movie, &QMovie::frameChanged, [=](int frameNumber) {
//        // GIF 动画执行三次就结束
//        if (frameNumber == m_movie->frameCount() - 1)
//        {
//            m_cycleNum++;
//            if(m_cycleNum >= maxCycleNum) {
//                m_cycleNum = 0;
//                m_movie->stop();
//                // 停止播放后，跳转到第一页
//                m_movie->jumpToFrame(0);
//            }
//        }
//    });

}

void TouchScreen::playGif(ScreenGestureType type)
{
    switch (type) {
    case ScreenGestureType::bottomEdgeSwipe:
        m_movie->setFileName(":/gif/resources/下边缘上划显示多任务视图.gif");
        break;
    case ScreenGestureType::rightEdgeSwipe:
        m_movie->setFileName(":/gif/resources/右边缘左划显示侧边栏.gif");
        break;
    case ScreenGestureType::fourFingersSwipeDown:
        m_movie->setFileName(":/gif/resources/四指向下划动显示搜索.gif");
        break;
    case ScreenGestureType::fourFingersHorizontalSwipe:
        m_movie->setFileName(":/gif/resources/四指水平滑动显示窗口切换.gif");
        break;
    default:
        break;
    }
    m_movie->stop();
    m_cycleNum = 0;
    m_movie->start();
}

void TouchScreen::stopGif()
{
    m_movie->stop();
//     停止播放后，跳转到第一页
    m_movie->jumpToFrame(0);
}

bool TouchScreen::autoSwitchTablet()
{
    QDBusInterface statusManager("com.kylin.statusmanager.interface",
                                 "/",
                                 "com.kylin.statusmanager.interface");
    QDBusMessage res = statusManager.call("get_modemonitor");
    if (res.type() == QDBusMessage::ErrorMessage) {
        qWarning() << res.errorName() << res.errorMessage();
    }
    else if(res.type() == QDBusMessage::ReplyMessage) {
        QDBusReply<bool> reply = res;
        return reply.value();
    }
    return false;
}

void TouchScreen::setAutoSwitchTablet(bool state)
{
    QDBusInterface statusManager("com.kylin.statusmanager.interface",
                                 "/",
                                 "com.kylin.statusmanager.interface");
    QDBusMessage res = statusManager.call("set_modemonitor",
                                          QVariant(state),
                                          QVariant("Touch Screen Plugin"),
                                          QVariant("Auto Switch Tablet Mode Button"));

    if (res.type() == QDBusMessage::ErrorMessage) {
        qWarning() << res.errorName() << res.errorMessage();
    }
    else if (res.type() == QDBusMessage::ReplyMessage) {
        QDBusReply<bool> reply = res;
        if (!reply) {
            qWarning() << "Set auto switch tablet mode error."
                       << "Function set_modemonitor of dbus servies com.kylin.statusmanager.interface"
                       << "return" << reply;
        }
    }
}

void TouchScreen::setAutoSwitchTabletBtn(bool checked)
{
    if (!m_autoSwitchTablet)
        return;
    if (m_autoSwitchTablet->isChecked() == checked)
        return;
    m_autoSwitchTablet->setChecked(checked);
}

//    // 下面这种方式可以加载 mp4 文件，但是控制面板加载会报 QXcbConnection 的错误
//    // 用 demo 加载插件不会出现此问题，暂时没有思路
//void TouchScreen::loadVideo()
//{
//    ui->video_widget->setStyleSheet("border-image: none; background: green; border:none");
//    player = new QMediaPlayer;
//    medialist = new QMediaPlaylist;
//    medialist->addMedia(QUrl::fromLocalFile("/home/pangyi/touchgesture/touchgesture/单指边缘-多任务视图.mp4"));
//    medialist->addMedia(QUrl::fromLocalFile("/home/pangyi/touchgesture/touchgesture/单指向下-显示桌面.mp4"));
//    medialist->setPlaybackMode(QMediaPlaylist::CurrentItemInLoop);
//    player->setPlaylist(medialist);
//    connect(player, QOverload<QMediaPlayer::Error>::of(&QMediaPlayer::error),
//          [=](QMediaPlayer::Error error){
//        qDebug() << error;
//    });
//    connect(player, &QMediaPlayer::stateChanged, this, &TouchscreenSettings::stateChanged);
//    connect(player, &QMediaPlayer::mediaStatusChanged, this, &TouchscreenSettings::mediaStatusChanged);

//    QVBoxLayout *videoLayout = new QVBoxLayout(ui->video_widget);

//    videoWidget = new QVideoWidget;
//    videoWidget->installEventFilter(this);
//    videoWidget->setFixedSize(400, 400);
//    videoLayout->setContentsMargins(0, 0, 0, 0);
//    videoLayout->addWidget(videoWidget);
//    player->setVideoOutput(videoWidget);
//}

//void TouchScreen::startVideo1()
//{
//    medialist->setCurrentIndex(1);
//    videoWidget->show();
//    player->play();
//}

//void TouchScreen::mediaStatusChanged(QMediaPlayer::MediaStatus status)
//{
//    switch (status) {
//    case QMediaPlayer::UnknownMediaStatus:
//        qDebug() << "媒体未知状况！";
//        break;
//    case QMediaPlayer::NoMedia:
//        qDebug() << "没有媒体文件！";
//        break;
//    case QMediaPlayer::BufferingMedia:
//        qDebug() << "正在缓冲媒体文件！";
//        break;
//    case QMediaPlayer::BufferedMedia:
//        qDebug() << "媒体文件缓冲完成！" << videoWidget->contentsMargins() << videoWidget->contentsRect() << videoWidget->geometry() << videoWidget->size() << videoWidget->baseSize();
//        break;
//    case QMediaPlayer::LoadingMedia:
//        qDebug() << "正在加载媒体！";
//        break;
//    case QMediaPlayer::StalledMedia:
//        qDebug() << "播放停滞！";
//        break;
//    case QMediaPlayer::EndOfMedia:
//        qDebug() << "播放结束！";
//        break;
//    case QMediaPlayer::LoadedMedia:
//        qDebug() << "媒体加载完成！";
//        break;
//    case QMediaPlayer::InvalidMedia:
//        qDebug() << "不可用的媒体文件！";
//        break;
//    default: break;
//    }
//}

//void TouchScreen::stateChanged(QMediaPlayer::State state)
//{
//    switch (state) {
//    case QMediaPlayer::StoppedState:
//        qDebug() << "停止状态！";
//        break;
//    case QMediaPlayer::PlayingState:
//        qDebug() << "播放状态！";
//        break;
//    case QMediaPlayer::PausedState:
//        qDebug() << "暂停状态！";
//        break;
//    default: break;
//    }
//}
